"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocumentationServiceImpl = void 0;
const tslib_1 = require("tslib");
const inversify_1 = require("inversify");
const path = tslib_1.__importStar(require("path"));
const types_1 = require("../types");
const format_1 = require("../model/format");
const kleur_1 = tslib_1.__importDefault(require("kleur"));
const code_format_1 = require("../model/code-format");
const comment_type_1 = require("../model/comment-type");
let DocumentationServiceImpl = class DocumentationServiceImpl {
    constructor(_freeCommentsExtractor, _microfocusCommentsExtractor, _tagCommentsParser, _xmlCommentsParser, _templateEngine, _outputStream) {
        this._freeCommentsExtractor = _freeCommentsExtractor;
        this._microfocusCommentsExtractor = _microfocusCommentsExtractor;
        this._tagCommentsParser = _tagCommentsParser;
        this._xmlCommentsParser = _xmlCommentsParser;
        this._templateEngine = _templateEngine;
        this._outputStream = _outputStream;
    }
    parse(files, outputDirectory, format, codeFormat, commentType) {
        console.log(`Output directory: ${outputDirectory}`);
        console.log(`Selected format: ${format}`);
        const commentsExtractor = this._getCommentsExtractor(codeFormat);
        const commentsParser = this._getCommentsParser(commentType);
        const exportedFiles = [];
        files.forEach(file => {
            this._parseFile(file, outputDirectory, format, commentsExtractor, commentsParser, exportedFiles);
        });
        const tableOfContents = format_1.TableOfContents.from(format);
        process.stdout.write(`Generating ${tableOfContents}... `);
        const outputContent = this._templateEngine.parseTableOfContents(format, exportedFiles);
        this._outputStream.write(outputDirectory, tableOfContents, outputContent);
        console.log(kleur_1.default.green('DONE'));
    }
    _parseFile(file, outputDirectory, format, commentsExtractor, commentsParser, exportedFiles) {
        const filePath = this._defineFilePath(file);
        const baseName = path.basename(filePath);
        const exportedFileName = `${baseName}.${format}`;
        process.stdout.write(`Generating ${baseName} documentation... `);
        const preDocumentation = commentsExtractor.extract(filePath);
        const documentation = commentsParser.parse(preDocumentation);
        const outputContent = this._templateEngine.parseDocumentation(format, documentation);
        this._outputStream.write(outputDirectory, exportedFileName, outputContent);
        console.log(kleur_1.default.green('DONE'));
        exportedFiles.push(exportedFileName);
    }
    _defineFilePath(file) {
        let filePath = file;
        if (!path.isAbsolute(file)) {
            filePath = path.join(process.cwd(), file);
        }
        return filePath;
    }
    _getCommentsExtractor(codeFormat) {
        switch (codeFormat) {
            case code_format_1.CodeFormat.FREE:
                return this._freeCommentsExtractor;
            case code_format_1.CodeFormat.MICROFOCUS:
                return this._microfocusCommentsExtractor;
            default:
                throw new Error(`Code format not supported: ${codeFormat}`);
        }
    }
    _getCommentsParser(commentType) {
        switch (commentType) {
            case comment_type_1.CommentType.TAG:
                return this._tagCommentsParser;
            case comment_type_1.CommentType.XML:
                return this._xmlCommentsParser;
            default:
                throw new Error(`Comment type not supported: ${commentType}`);
        }
    }
};
DocumentationServiceImpl = tslib_1.__decorate([
    inversify_1.injectable(),
    tslib_1.__param(0, inversify_1.inject(types_1.TYPES.FreeCommentsExtractor)),
    tslib_1.__param(1, inversify_1.inject(types_1.TYPES.MicrofocusCommentsExtractor)),
    tslib_1.__param(2, inversify_1.inject(types_1.TYPES.TagCommentsParser)),
    tslib_1.__param(3, inversify_1.inject(types_1.TYPES.XmlCommentsParser)),
    tslib_1.__param(4, inversify_1.inject(types_1.TYPES.TemplateEngine)),
    tslib_1.__param(5, inversify_1.inject(types_1.TYPES.FileOutputStream)),
    tslib_1.__metadata("design:paramtypes", [Object, Object, Object, Object, Object, Object])
], DocumentationServiceImpl);
exports.DocumentationServiceImpl = DocumentationServiceImpl;
